@@ -2,10 +2,10 @@ mod app;
 mod constants;
 mod context;
 mod data;
+mod dependency;
 mod error;
 mod function;
 mod meta;
-mod package;
 mod pipe;
 mod utils;
 
@@ -13,9 +13,9 @@ pub use app::*;
 pub(crate) use constants::*;
 pub(crate) use context::*;
 pub(crate) use data::*;
+pub use dependency::*;
 pub(crate) use error::*;
 pub(crate) use function::*;
-pub use package::*;
 pub(crate) use pipe::*;
 
 pub trait Entity {
